
# q-expansion of newform 9360.2.a.bs, downloaded from the LMFDB on 23 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 9360
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [1], [1], [1], [-1], [-5], [0], [-3], [-2], [6], [1], [-7], [-2], [6], [-5], [4], [-7], [-12], [1], [10], [-11], [12], [-13], [-1], [-8], [6], [-3], [6], [-2], [-4], [-8], [-12], [-1], [-17], [12], [4], [1], [2], [-22], [-10], [-11], [-6], [17], [12], [0], [-4], [8], [8], [4], [-3], [-23], [2], [24], [6], [-16], [-6], [20], [14], [-6], [4], [14], [-3], [-6], [-24], [-18], [32], [14], [-15], [8], [14], [12], [-36], [10], [-18], [-24], [10], [29], [-18], [4], [-26], [-8], [-24], [16], [-25], [1], [-15], [-13], [-3], [17], [-23], [23], [-5], [-42], [10], [-32], [15], [26], [24], [-8], [-22], [-14], [33], [42], [5], [3], [-12], [-34], [-42], [-5], [-8], [23], [-6], [-32], [6], [-24], [13], [-21], [6], [38], [-30], [26], [15], [4], [44], [-8], [-4], [12], [2], [17], [-6], [-36], [31], [-18], [26], [10], [-20], [-4], [-35], [4], [26], [25], [16], [42], [-46], [-19], [7], [-31], [-45], [-18], [22], [-6], [-2], [-53], [14], [28], [-7], [-57], [12], [-41], [58], [-9], [-16], [20], [42], [44], [3], [10], [-44], [3], [20], [20], [10], [19], [-29], [26], [-10], [-21], [-56], [14], [29], [16], [-12], [4], [34], [-54], [-4], [31], [-32], [47], [28], [14], [-19], [-55], [64], [31], [-18], [-66], [16], [-11], [2], [33], [3], [5], [27], [-63], [-40], [36], [19], [-41], [2], [42], [48], [-4], [-32], [-10], [42], [-18], [24], [-4], [55], [6], [-18], [32], [9], [-27], [-11], [-46], [-34], [73], [64], [23], [28], [76], [60], [10], [-22], [-36], [52], [-64], [32], [19], [-22], [-46], [-34], [48], [-40], [45], [10], [-72], [-38], [30], [12], [45], [9], [-9], [66], [57], [58], [-33], [44], [-13], [-40], [-34], [-28], [-19], [51], [-33], [63], [46], [50], [83], [39], [-16], [-24], [15], [36], [34], [80], [64], [-75], [14], [-24], [22], [52], [6], [-14], [30], [-50], [-60], [48], [-12], [54], [44], [-12], [30], [-48], [81], [-20], [61], [60], [-16], [1], [-1], [-60], [-19], [47], [-49], [-8], [56], [29], [18], [52], [56], [-14], [-75], [19], [-71], [-62], [-16], [0], [-10], [0], [-75], [88], [71], [-16], [-7], [-3], [-22], [-88], [5], [-52], [-36], [-28], [-42], [-44], [-30], [-26], [22], [39], [57], [51], [56], [-18], [-58], [73], [37], [-40], [-3], [16], [1], [-15], [36], [-17], [86], [-19], [36], [2], [26], [-6], [62], [-52], [-24], [-16], [-86], [12], [73], [65], [18], [28], [38], [68], [-77], [-52], [67], [-8], [87], [30], [-68], [74], [-6], [-20], [-50], [-18], [66], [-16], [-76], [-48], [33], [-16], [24], [49], [62], [9], [-8], [26], [36], [-77], [-38], [-22], [-55], [47], [-40], [-51], [103], [-58], [50], [-53], [19], [-76], [5], [-27], [-17], [-64], [-20], [-45], [-22], [-28], [-13], [50], [29], [-64], [62], [86], [-88], [49], [-39], [-74], [52], [36], [45], [-67], [78], [23], [-52], [20], [16], [-24], [78], [78], [7], [-8], [84], [50], [30], [86], [66], [-94], [-42], [-23], [-76], [-98], [6], [21], [-46], [30], [68], [-56], [-48], [-7], [40], [32], [-55], [89], [65], [-17], [-36], [36], [-100], [-48], [-20], [-75], [94], [86], [-99], [-106], [88], [27], [64], [-76], [84], [-92], [-38], [-28], [117], [44], [-77], [-26], [13], [75], [-64], [-86], [-82], [-94], [-86], [25], [84], [-36], [-53], [-99], [-44], [-78], [47], [50], [-106], [-45], [-93], [14], [78], [48], [-9], [-119], [-6], [-60], [30], [-30], [59], [90], [-12], [30], [60], [-6], [103], [53], [-20], [114], [9], [-16], [-28], [20], [72], [-100], [-90], [14], [-98], [38], [39], [-94], [-9], [0], [103], [-4], [84], [-90], [22], [70], [-58], [-83], [-11], [26], [58], [-11], [-57], [-29], [51], [92]]
